
import * as api from "./api";
import journey_wrapper from "./types/wrapper";
import { migrator } from "./types/migration";
import { focus_day, focus_leg, nav_mouseleave, nav_mousemove } from "./helper/nav";
import { set_search_set_results } from "./helper/api";
import toast from "./types/toast";
import search_drawer from "./types/search_drawer";
import { to_xy } from "./types/geom";

Vue.component("l-map", window.Vue2Leaflet.LMap);
Vue.component("l-tile-layer", window.Vue2Leaflet.LTileLayer);
Vue.component("l-marker", window.Vue2Leaflet.LMarker);
Vue.component("l-icon", window.Vue2Leaflet.LIcon);
Vue.component("l-popup", window.Vue2Leaflet.LPopup);
Vue.component("l-tooltip", window.Vue2Leaflet.LTooltip);
Vue.component("l-polyline", window.Vue2Leaflet.LPolyline);
Vue.component("l-control-scale", window.Vue2Leaflet.LControlScale);

const app = new Vue({
  el: "#app",
  data: {
    journey: new journey_wrapper(),
    search_drawer: new search_drawer(),
    useDay: false,
    toast: new toast(),
    lang: {
      format: "ddd D MMM",
      formatLocale: {
        firstDayOfWeek: 1,
      },
      monthBeforeYear: true,
    },
  },
  methods: {
    start_journey: function () { window.location.href = "/" + this.journey.id },

    generate_rotation: function (index, list) {
      if (index < 0 || index >= list.length) return 0;
      const c0 = to_xy(list[(index == 0) ? index : (index - 1)])
      const c1 = to_xy(list[(index == (list.length - 1)) ? index : (index + 1)])
      const brng = Math.atan2(c1[1] - c0[1], c1[0] - c0[0])- Math.PI / 2;
      const flip =  (brng>-Math.PI/2)?'transform: scale(1,1);': 'transform: scale(1, -1);';
      const rot = `rotate:${brng}rad;`
      return `${rot}${flip}`;

    },
    generate_marker: function (item, fcolor) {
      return `
			<div style="position: absolute;top: -30px;left: -6px;">
  			<i class=" fa fa-${api.icon_type(item) || "star"}  fa-lg icon-white" style="position: absolute;text-align:center; width:24px;height:24px; line-height:1.5em"></i>
			<svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 24 36" width="24" height="36">
              <circle cx="12" cy="12" r="12" fill="${fcolor || item.color || "blue"}"/>
              <polygon points="4,12 20,12 12,36" fill="${fcolor || item.color || "blue"}" /></svg>`
    },
    generate_icon: function (item, fcolor = "", styling = "", classes = "") {
      return `<i class="fa fa-${api.icon_type(item) || "star"} fa-2x ${classes}"  style="${styling}; color:${fcolor || "white"}; text-align:center; align-content:center;"></i>`;
    },

    refreshTextAreaHeight: function (e) {
      e.target.style['height'] = 'auto';
      e.target.style['height'] = e.target.scrollHeight + 'px';
      e.target.style['max-height'] = "100%";
    },

    onMapClick(e) {
      if (this.search_drawer.query.addmarker) {
        const newMarker = {
          latlon: [e.latlng.lat, e.latlng.lng],
          sname: this.search_drawer.query.query,
          type: this.search_drawer.query.type,
        };
        this.search_drawer.click_item(this.journey,newMarker)
      }
    },
  },
  created: function () {
    set_search_set_results((r) => { this.search_drawer.results(r); });
    this.nav_mouseleave = nav_mouseleave;
    this.nav_mousemove = nav_mousemove;
    this.focus_day = focus_day;
    this.focus_leg = focus_leg;

    this.save_data = api.throttle(() => {
      api.save(this.journey.id, this.journey.data);
    }, 2000);

    this.pressed_prev_next = api.throttle((dir_prev) => {
      if (this.useDay && dir_prev) return this.journey.day_prev();
      if (!this.useDay && dir_prev) return this.journey.leg_prev();
      if (this.useDay && !dir_prev) return this.journey.day_next();
      if (!this.useDay && !dir_prev) return this.journey.leg_next();
    }, 250)

    window.addEventListener("keydown", (e) => {
      switch (e.key) {
        case "ArrowLeft": return this.pressed_prev_next(true)
        case "ArrowRight": return this.pressed_prev_next(false)
        default: return;
      }
    });

    api.load(this.journey.id).then((r) => {
      app.journey.data = migrator(r)
      setTimeout(() => focus_leg(this.journey) && focus_day(this.journey) && this.$refs.map.mapObject.keyboard.disable(), 50);

      ;
      this.search_drawer.refresh_map = () => { this.$refs.map.mapObject.invalidateSize() }
      this.search_drawer.get_leg = () => { return this.journey.leg_get() }
      this.search_drawer.get_bb = () => {
        if (!this.$refs.map) return []
        const bounds = this.$refs.map.mapObject.getBounds();
        return [[bounds.getSouthWest().lng, bounds.getSouthWest().lat],
        [bounds.getNorthEast().lng, bounds.getNorthEast().lat]]
      }
    });
  },
  watch: {
    journey: {
      handler: function (ndata, odata) {
        if (this.journey.edit) this.save_data();
        window.location.hash = `#${this.journey.sel_leg}_${this.journey.sel_day}`
      },
      deep: true,
    },
  },
});
